/***************************************************************************
    comm.c  -  Communication with UPS
    -----------------------
    begin                : pi lut 11 09:59:54 CET 2000
    copyright            : (C) 2000 by Tomasz Fiszer
    email                : profis@kki.net.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <termios.h>	// Needed by TTY(s)
#include <fcntl.h>
#include <signal.h>
#include <time.h>
#include "utils.h"
#include "defs.h"
#include "ups.h"
#include "struct.h"


int LE_BIT	= TIOCM_LE;
int DTR_BIT 	= TIOCM_DTR;
int RTS_BIT	= TIOCM_RTS;
int ST_BIT	= TIOCM_ST;
int SR_BIT	= TIOCM_SR;
int CTS_BIT	= TIOCM_CTS;
int CD_BIT	= TIOCM_CD;
int RNG_BIT	= TIOCM_RNG;
int DSR_BIT	= TIOCM_DSR;

static int ReadTimeout = 0;
static int iTestBits   = 0;

void ReadTimeoutAlarm(int);
void SendByte(UPS_INFO *, unsigned char);
void ReceiveByte(UPS_INFO *, unsigned char *);

//
// Otwiera port szeregowy i konfiguruje
//
int SetupSerial(UPS_INFO *upsinfo, int iKpwr)
{
    int		iError;
    if (upsinfo->szSerialDevice == NULL)
	return _ERROR_BAD_PARAM;

    upsinfo->iSerial = open(upsinfo->szSerialDevice, O_RDWR | O_NOCTTY | O_APPEND/* | O_SYNC*/);
    if (upsinfo->iSerial < 0)
    {
	char	szErr[256];
	sprintf(szErr, "Can't open serial port: %s", upsinfo->szSerialDevice);
	PrintError(szErr, errno, 1);
	PrintToLog("Can't open serial port (%s)!\n\tError description: %s\n",
	    upsinfo->szSerialDevice, strerror(errno));
	return _ERROR_SETUP_SERIAL;
    }
    else
    {
	int	iFlag = fcntl(upsinfo->iSerial, F_GETFL, 0);
        fcntl(upsinfo->iSerial, F_SETFL, iFlag & ~(O_NDELAY));
    }
    iError = CreateLockFile(upsinfo, iKpwr);
    if (iError != _SUCCESS_LOCKFILE)
    {
	close(upsinfo->iSerial);
	upsinfo->iSerial = -1;
	return _ERROR_LOCKFILE;
    }
    tcgetattr(upsinfo->iSerial, &oldtio);
    // Setting proc. for termination signals
    signal(SIGHUP, Terminate);
    signal(SIGINT, Terminate);
    signal(SIGTERM, Terminate);
    signal(SIGKILL, Terminate);
    
    newtio.c_cflag = BAUDRATE | CRTSCTS | CS8 | CLOCAL | CREAD;
    newtio.c_iflag = IGNPAR | IGNBRK;
    newtio.c_oflag = 0;
    newtio.c_lflag = 0;
    newtio.c_ispeed= BAUDRATE;
    newtio.c_ospeed= BAUDRATE;
    newtio.c_cc[VMIN] = 0;
    newtio.c_cc[VTIME] = 10;
    tcflush(upsinfo->iSerial, TCIFLUSH);
    tcsetattr(upsinfo->iSerial, TCSANOW, &newtio);

    //ioctl(upsinfo->iSerial, TIOCMBIC, &DTR_BIT);
    //ioctl(upsinfo->iSerial, TIOCMBIC, &DSR_BIT);
    //ioctl(upsinfo->iSerial, TIOCMBIC, &RTS_BIT);
    //ioctl(upsinfo->iSerial, TIOCMBIC, &CTS_BIT);

    return 0;
}

//
// Send byte to current setup serial port
//
void SendByte(UPS_INFO *Upsinfo, unsigned char ucByte)
{
    usleep(_SR_TIMEOUT);
    tcflush(Upsinfo->iSerial, TCIOFLUSH);
    write(Upsinfo->iSerial, &ucByte, 1);
TRACEF2(("SEND BYTE: %d SERIAL: %s\n", ucByte, Upsinfo->szSerialDevice));
}

//
// Receive byte from current setup serial port
//
void ReceiveByte(UPS_INFO *Upsinfo, unsigned char *ucRec)
{
    unsigned char ucByte = '\0';
    usleep(_SR_TIMEOUT);
    signal(SIGALRM, ReadTimeoutAlarm);
    alarm(2);
    ReadTimeout = 0;
    read(Upsinfo->iSerial, &ucByte, 1);
    if (ReadTimeout > 0)
    {
TRACEF2(("RECEIVE BYTE: %d (TIMEOUT(%d)) SERIAL: %s\n", ucByte, ReadTimeout, Upsinfo->szSerialDevice));
	*ucRec = '\0';
	Upsinfo->iLastError = _ERROR_COMM_TIMEOUT;
	iTestBits = 1;
    }
    else
    {
TRACEF2(("RECEIVE BYTE: %d SERIAL: %s\n", ucByte, Upsinfo->szSerialDevice));
	if (iTestBits != 0)
	{
	    iTestBits = 0;
	}
	*ucRec = ucByte;
	Upsinfo->iLastError = SUCCESS;
    }
    alarm(0);
    signal(SIGALRM, SIG_IGN);
}

//
// Read Timeout function for signal handling
//
void ReadTimeoutAlarm(int iSig)
{
    ReadTimeout++;
}

