/***************************************************************************
    powersoft.c - program for UPS's monitoring
    -----------------------------------------------
    begin	: pia lut 11 09:59:54 CET 2000
    copyright	: (C) 2001 by Tomasz Fiszer
    email	: tfiszer@ever.com.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <fcntl.h>
#include <time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include "defs.h"
#include "utils.h"
#include "strman.h"
#include "conf.h"
#include "comm.h"
#include "ups.h"
#include "threads.h"
#include "struct.h"

// Global variables definition
extern int 	errno;
char		szRetBuffer[1024],	//  
		szBuffer[256];		//  Global string buffers
char		szConfigPath[4096]	= POWERSOFTCONF;
// Global Structures		
static UPS_INFO UpsInfo;
FILE		*fProcfile = NULL;
FILE		*fLogfile = NULL;

//
// Global functions definitions
//
void	WelcomeDisplay();
void	InitAll();
int	RetriveConfiguration(char *, UPS_INFO *);
void	Terminate(int);
char	*GetTypeUpsName(UPS_INFO *);
void	CleanThreads();
void	CheckArgs(char **lpszArg, int iArg);

//
// Main function
//
int main(int argc, char** argv)
{
    int iError, icnt;
    unsigned char ucTest;

    unlink(PS_FIFO_SERVER);
    unlink(PS_FIFO_CLIENT);
    PrintToLog("Rozpoczecie pracy programu na %s.\n", getenv("USER"));
    WelcomeDisplay();

    if (argc >= 2)
    {
	for (icnt = 1; icnt < argc; icnt++)
	    CheckArgs(argv, icnt);
    }   

    if (!CheckSuperUser())
    {
	// Exit here because we need "Root" access rights
	PrintToLog(
	    "Do uruchomienia tego programu wymagane sa prawa "
	    "administratora!", getenv("USER"));
	_exit(0);
    }

    InitAll();
    if (RetriveConfiguration(szConfigPath, &UpsInfo) != 0)
    {
	PrintToLog(
	    "\tNalezy wlasciwie skonfigurowac program do poprawnej pracy.\n"
	    "\tPlik konfiguracyjny nalezy umiescic w \"%s\"\n",
	    szConfigPath);
	PrintError("Plik konfiguracyjny nie zostal znaleziony!\n", 0, 0);
	return EXIT_FAILURE;
    }
    DeleteLockFile(&UpsInfo);

TRACE;
    iError = SetupSerial(&UpsInfo, 0);
    if (iError != 0)
	return EXIT_FAILURE;

    if (!Code(&UpsInfo))
    {
	ReceiveByte(&UpsInfo, &ucTest);
	ReceiveByte(&UpsInfo, &ucTest);
	tcflush(UpsInfo.iSerial, TCIOFLUSH);
	if (!Code(&UpsInfo))
	{
	    tcflush(UpsInfo.iSerial, TCIOFLUSH);
	    PrintToLog("\tNie wykryto zasilacza!\n");
	    PrintError("Nie wykryto zasilacza!\n", 0, 0);
	    return EXIT_FAILURE;
	}
	else
	{
	    SendByte(&UpsInfo, 208);
	    ReceiveByte(&UpsInfo, &ucTest);
	    ReceiveByte(&UpsInfo, &ucTest);
	    usleep(250000);
    	    if (Code(&UpsInfo))
	    {
		SendByte(&UpsInfo, 173);
		ReceiveByte(&UpsInfo, &ucTest);
		if (ucTest < 67 || ucTest > 82)
		{
		    PrintToLog("\tNierozpoznany typ zasilacza!\n");
		    PrintError("Nierozpoznany typ zasilacza lub blad komunikacji!\n", 0, 0);
		    return EXIT_FAILURE;
		}else
		{
		    printf("\nTyp zasilacza: %s\n\n", GetUpsName((int)ucTest));
		    printf("Rozpoczecie monitoringu...\n\n");
		}
	    }else
	    {
		tcflush(UpsInfo.iSerial, TCIOFLUSH);
		PrintToLog("\tNie wykryto zasilacza!\n");
		PrintError("Nie wykryto zasilacza!\n", 0, 0);
		return EXIT_FAILURE;
	    }
	}	
    }
    else
    {
	SendByte(&UpsInfo, 208);
	ReceiveByte(&UpsInfo, &ucTest);
	ReceiveByte(&UpsInfo, &ucTest);
	usleep(250000);
	if (Code(&UpsInfo))
	{
	    SendByte(&UpsInfo, 173);
	    ReceiveByte(&UpsInfo, &ucTest);
	    if (ucTest < 67 || ucTest > 82)
	    {
		PrintToLog("\tNierozpoznany typ zasilacza!\n");
		PrintError("Nierozpoznany typ zasilacza lub blad komunikacji!\n", 0, 0);
		return EXIT_FAILURE;
	    }else
	    {
		printf("\nTyp zasilacza: %s\n\n", GetUpsName((int)ucTest));
		printf("Rozpoczecie monitoringu...\n\n");
	    }
	}
	else
	{
	    tcflush(UpsInfo.iSerial, TCIOFLUSH);
	    PrintToLog("\tNie wykryto zasilacza!\n");
	    PrintError("Nie wykryto zasilacza!\n", 0, 0);
	    return EXIT_FAILURE;
	}
    }
TRACE;
    switch(fork())
    {
	case 0: 
	    setsid();
	    break;
	case -1:
	    PrintError("Sorry! I can't fork.", 0, 0);
	    PrintToLog(
		"I can't fork!\n"
		"\tThere can be too many processes?\n");
	    exit(1);
	default:
	    exit(0);
    }
    
    OpenLog();
    CreatePID();
    sleep(1);
    
    if (CreateLockFile(&UpsInfo, 0) == _ERROR_LOCKFILE)
    {
	PrintError("Failed to reacquire lock file on TTY device!\n", 0, 0);
	PrintToLog("Unable to reacquire lock file on TTY device!\n");
	Terminate(0);
    }
    
    InitSystemSignals();

    UpsInfo.ulUpsStateCfg = 0;
    UpsInfo.ulTimeToShutdown1 = UpsInfo.ulUserShutdownTimeout;
    UpsInfo.ulTimeToShutdown2 = UpsInfo.ulUserShutdownTimeout;
    //CreateThread(MainUPSLoop, &UpsInfo);
    MainUPSLoop(&UpsInfo);
    
    StandbyForTermination();
    CloseLog();
    PrintToLog("Zakonczenie programu z powodu bledu!\n");
    PrintError("Zakonczenie programu z powodu bledu!\n", 0, 0);
    Terminate(0);
    return EXIT_SUCCESS;
}

//
// Initialization of all variables
//
void InitAll()
{
TRACE;
    memset(&UpsInfo, 0, sizeof(UpsInfo));
TRACE;
}

//
// Text displayed when program starts
//
void WelcomeDisplay()
{
TRACE;
    printf(
	"PowerSoft v%s\n\t%s\n"
	"Kompilacja %s (%s)\n", 
	PRIMARY_VER, PRIMARY_COPYRIGHT, __DATE__, __TIME__);
TRACE;
};

//
// Getting all configuration data
//
int RetriveConfiguration(char szConfig[4096], UPS_INFO *upsinfo)
{
    int		iErrorConfig = 0,
		icnt;
    char	szBuf[4096] = "";
TRACE;
    if (szConfig == NULL)
	strcpy(szConfig, POWERSOFT_CONFIG_FILE);

    iErrorConfig = GetConfig_String(szConfig, "SerialPort", 
	"/dev/ttyS1", upsinfo->szSerialDevice);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;
    iErrorConfig = GetConfig_String(szConfig, "LockFile", 
	LOCK_DEFAULT, upsinfo->szLockFilePath);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;
    //
    // Messages
    //
    iErrorConfig = GetConfig_String(szConfig, "Msg_PowerFailure_Text", 
	"Power Failure!", upsinfo->szMsgPowerFailure);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;
    iErrorConfig = GetConfig_String(szConfig, "Msg_PowerOK_Text", 
	"Power OK", upsinfo->szMsgPowerOK);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;
    iErrorConfig = GetConfig_String(szConfig, "Msg_Standby_Text", 
	"UPS standing by!", upsinfo->szMsgStandby);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;
    iErrorConfig = GetConfig_String(szConfig, "Msg_Timeout_Text", 
	"", upsinfo->szMsgTimeout);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;
    iErrorConfig = GetConfig_String(szConfig, "Msg_UPSError_Text", 
	"", upsinfo->szMsgUPSError);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;
    iErrorConfig = GetConfig_String(szConfig, "Msg_Shutdown_Text", 
	"System will be shutdown in 30 seconds!", upsinfo->szMsgShutdown);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;
    iErrorConfig = GetConfig_String(szConfig, "Msg_Info_Interval", 
	"30", szBuf);
    upsinfo->uiMsgInterval = atol(szBuf);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;
    iErrorConfig = GetConfig_String(szConfig, "Msg_Shutdown_Interval", 
	"20", szBuf);
    upsinfo->uiMsgShutdownInterval = atol(szBuf);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;
    iErrorConfig = GetConfig_String(szConfig, "Msg_Enabled", 
	"1", szBuf);
    upsinfo->uiMsgEnabled = atol(szBuf);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;
    //
    // Commands
    //
    iErrorConfig = GetConfig_String(szConfig, "Cmd_PowerFailure", 
	"", upsinfo->szCmdPowerFailure);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;
    iErrorConfig = GetConfig_String(szConfig, "Cmd_PowerOK", 
	"", upsinfo->szCmdPowerOK);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;
    iErrorConfig = GetConfig_String(szConfig, "Cmd_Standby", 
	"", upsinfo->szCmdStandby);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;
    iErrorConfig = GetConfig_String(szConfig, "Cmd_Timeout", 
	"", upsinfo->szCmdTimeout);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;
    iErrorConfig = GetConfig_String(szConfig, "Cmd_UPSError", 
	"", upsinfo->szCmdUPSError);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;
    iErrorConfig = GetConfig_String(szConfig, "Cmd_Shutdown", 
	"", upsinfo->szCmdShutdown);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;
    iErrorConfig = GetConfig_String(szConfig, "Cmd_Shutdown_Interval", 
	"10", szBuf);
    upsinfo->uiCmdShutdownInterval = atol(szBuf);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;
    iErrorConfig = GetConfig_String(szConfig, "Cmd_Enabled", 
	"0", szBuf);
    upsinfo->uiCmdEnabled = atol(szBuf);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;

    ClearFrontSpaces(upsinfo->szCmdPowerFailure);
    ClearFrontSpaces(upsinfo->szCmdPowerOK);
    ClearFrontSpaces(upsinfo->szCmdStandby);
    ClearFrontSpaces(upsinfo->szCmdTimeout);
    ClearFrontSpaces(upsinfo->szCmdUPSError);
    ClearFrontSpaces(upsinfo->szCmdShutdown);

    ClearDoubleQuote(upsinfo->szCmdPowerFailure);
    ClearDoubleQuote(upsinfo->szCmdPowerOK);
    ClearDoubleQuote(upsinfo->szCmdStandby);
    ClearDoubleQuote(upsinfo->szCmdTimeout);
    ClearDoubleQuote(upsinfo->szCmdUPSError);
    ClearDoubleQuote(upsinfo->szCmdShutdown);

    // User time to shutdown
    iErrorConfig = GetConfig_String(szConfig, "ShutdownTimeOut", 
	"300", szBuf);
    upsinfo->ulUserShutdownTimeout = atol(szBuf);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;

    iErrorConfig = GetConfig_String(szConfig, "RecoveryTime", 
	"5", szBuf);
    upsinfo->ulRecoveryTime = atol(szBuf);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;

    iErrorConfig = GetConfig_String(szConfig, "ShutdownToStandByTime",
	"9999", szBuf);
    upsinfo->uiStandbyTimeout = atol(szBuf);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;

    iErrorConfig = GetConfig_String(szConfig, "AutotestEnabled",
	"1", szBuf);
    upsinfo->uiAutotestEnabled = atol(szBuf);
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;

    //
    // Autotest time in minutes
    // We must convert it on seconds
    iErrorConfig = GetConfig_String(szConfig, "AutotestInterval",
	"180", szBuf);
    upsinfo->ulAutotestInterval = atol(szBuf);
    upsinfo->ulAutotestInterval *= 60;
    if (iErrorConfig == _ERROR_CONFIG)
	return _ERROR_CONFIG;

    iErrorConfig = GetConfig_String(szConfig, "NoLogin_Enabled",
	"1", szBuf);
    upsinfo->uiNoLoginEnabled = atol(szBuf);

    // Serial device path
    ClearFrontSpaces(upsinfo->szSerialDevice);
    // Lock file path
    ClearFrontSpaces(upsinfo->szLockFilePath);
    // Now we have only path to directory of lock files
    icnt = GetNumArguments(upsinfo->szLockFilePath, '"', "/");
    if (icnt)
    {
	GetArgumentFromString(upsinfo->szSerialDevice, szBuf, icnt, '"', "/");
	strcat(upsinfo->szLockFilePath, "/");
	strcat(upsinfo->szLockFilePath, LOCK_NAME);
	strcat(upsinfo->szLockFilePath, szBuf);
    }

    ClearFrontSpaces(upsinfo->szMsgPowerFailure);
    ClearFrontSpaces(upsinfo->szMsgPowerOK);
    ClearFrontSpaces(upsinfo->szMsgStandby);
    ClearFrontSpaces(upsinfo->szMsgTimeout);
    ClearFrontSpaces(upsinfo->szMsgUPSError);
    ClearFrontSpaces(upsinfo->szMsgShutdown);

    return 0;
}

//
// Clean from opened files
//
void CleanupFiles()
{
    if (fLogfile != NULL) 
    {
	fflush(fLogfile);
	fclose(fLogfile);
    }
    if (fProcfile != NULL) 
    {
	fflush(fProcfile);
	fclose(fProcfile);
    }
}

//
// Terminate program from signal
//
void Terminate(int iSig)
{
    CleanupFiles();
    if (UpsInfo.iSerial != -1)
    {
	tcflush(UpsInfo.iSerial, TCIFLUSH);
	tcsetattr(UpsInfo.iSerial, TCSANOW, &oldtio);
	close(UpsInfo.iSerial);
    }
    CleanThreads();
    DeleteLockFile(&UpsInfo);
    _exit(0);
}

//
// Terminate threads
//
void TerminateThread()
{
    RestoreSystemSignals();
    _exit(0);
}

//
// Cleaning threads
//
void CleanThreads()
{
    int	i, iStatus;

    for (i = 0; i < MAXTHREADS; i++)
    {
	if (iThreadPID[i] == 0)
	    break;
	kill(iThreadPID[i], SIGTERM);
	waitpid(iThreadPID[i], &iStatus, 0);
    }
}

//
// Checking arguments
//
void CheckArgs(char **lpszArg, int iArg)
{
    if (strcmp(lpszArg[iArg], "--help") == 0)
    {
	printf("\nUzycie programu:\n");
	printf("\t./powersoft [OPTION]...\n");
	printf("\tOptions:\n");
	printf("\t  --help              - wyswietla niniejszy ekran\n");
	printf("\t  --configfile <path> - uzywa pliku konfiguracji <path>\n");
	printf("\n\n");
	exit(0);
    }
    if (strcmp(lpszArg[iArg], "--configfile") == 0)
    {
	printf("\nUzycie pliku konfiguracyjnego z : %s\n", lpszArg[iArg+1]);
        strcpy(szConfigPath, lpszArg[iArg+1]);
        return;
    }
}

