/***************************************************************************
    psstat.c  -  Panel pomiarowy dla PowerSoft
    ------------------------
    begin	: pia lut 11 09:59:54 CET 2000
    copyright	: (C) 2000 by Tomasz Fiszer
    email	: profis@kki.net.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <fcntl.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <curses.h>
#include <panel.h>
#include <menu.h>
#include <setjmp.h>
#include <errno.h>

#include "defs.h"
#include "struct.h"
#include "utils.h"
#include "strman.h"
#include "conf.h"
#include "fifo.h"
#include "threads.h"

typedef 	unsigned int UINT;
// Global variables definition
extern int 	errno;
char		szRetBuffer[1024],	//  
		szBuffer[256];		//  Global string buffers
// Global Structures		
static UPS_INFO		UpsInfo;
static UPS_INFO_FIFO	UpsInfoFifo;
FILE		*fProcfile	= NULL;
FILE		*fLogfile	= NULL;
static int	HasColors	= FALSE;
static int	iConnected	= FALSE;
static int	bLoopBusy	= FALSE;
WINDOW		*MenuWin;
PANEL		*MenuPanel;
WINDOW		*MainWin;
PANEL		*MainPanel;
WINDOW		*BarWin;
PANEL		*BarPanel;
WINDOW		*StatusWin;
PANEL		*StatusPanel;

// Global function definitions
void	WelcomeDisplay();
void	InitAllVariables();

int	InitGraphics();
int	CloseCurses();
void	ClearWindow(WINDOW *, int, int, int, int);
void	UpdateAll();
int	DisplayMainMenu(int);
int	SubMenus(int *, int);
static int SubMenuDriver(int);
ITEM	**LoadMenuItems(ITEM **, int);
void	MsgBox(char *);
void	RefreshWindows();
void	PrintWinInfo(WINDOW *, int, int, UPS_INFO_FIFO *);
void	ProgressBar(WINDOW *, int, int, int, int, int, int, int);
void	SetWindowText(WINDOW *, int, int, char *);
int	ExecuteCommand(int);
void	ConnectToUPS();
char	*Spaces(char *, int);
void	PrintInformation(UPS_INFO_FIFO *);
void	Terminate(int);
void	TerminateThread();
char	*GetTypeUpsName(UPS_INFO_FIFO *);
char	*GetUpsState(UPS_INFO_FIFO *);
char	*GetAccuString(char *, unsigned long);
char	*GetAccuCapacityString(char *, UPS_INFO_FIFO);
char	*GetTimeString(char *, unsigned long);
int	GetAccuCapacity(UPS_INFO_FIFO);
void	MainThread(int);
void	Autotest();
void	SetStatusText(char *);
char	*Warning(char *, UPS_INFO_FIFO);
void	PrintDebugString(char *, ...);

#define RESTORE_BUFFER		5
#define HMENU_ENTRIES		2

//
// Coordinates of Menu window
//
#define MENUWIN_X		0
#define MENUWIN_Y		0
#define MENUWIN_COLS		80
#define MENUWIN_ROWS		1
//
// Coordinates of progress bar window
//
#define BARWIN_X		0
#define BARWIN_Y		14
#define BARWIN_COLS		80
#define BARWIN_ROWS		10
#define BARWIN_TITLE		" Parametry pracy zasilacza "
//
// Coordinates of main window
//
#define MAINWIN_X		0
#define MAINWIN_Y		1
#define MAINWIN_COLS		80
#define MAINWIN_ROWS		13
#define MAINWIN_TITLE		" Dane zasilacza "
//
// Coordinates of status window
//
#define STATUSWIN_X		0
#define STATUSWIN_Y		24
#define STATUSWIN_COLS		80
#define STATUSWIN_ROWS		1

// Sub menus (returned commands)
// other (less than those) are indexes 
// of current selected item
#define SM_CONTINUE		0x1000
#define SM_QUITMENU		0x2000

//
// Direction of progress bar 
//
#define BAR_HTYPE		0x0001
#define BAR_VTYPE		0x0002

#define MENU_DIV		0x0010

//
// Function:	PrintDebugString
// Comment:	Printing debug string to stdout only when compiled with "USE_DBGF"
//
void PrintDebugString(char *lpszFormat, ...)
{
#ifdef _USE_DBGF
    va_list	ap;
    char	szMsgBuffer[512];
    va_start(ap, lpszFormat);
    vsprintf(szMsgBuffer, lpszFormat, ap);
    va_end(ap);
    printf(szMsgBuffer);
#endif//_USE_DBGF
}

void RefreshWindows()
{
    hide_panel(MenuPanel);
    hide_panel(MainPanel);
    hide_panel(StatusPanel);
    show_panel(MenuPanel);
    show_panel(MainPanel);
    show_panel(StatusPanel);
    UpdateAll();
}

//
// Initialization of ncurses graphics
//
int InitGraphics()
{
    if (initscr() == NULL)
    {
	PrintDebugString("initscr() failed!\n");
	return FAILURE;
    }
    
    savetty();
    
    if (LINES < 25 || COLS < 80)
    {
	PrintError("You must have at least 25 lines and 80 columns!\n", 0, 0);
	return FAILURE;
    }
    
    if (cbreak() == ERR)
    {
	PrintDebugString("cbreak() == ERR !\n");
	return FAILURE;
    }
    if (noecho() == ERR)
    {
	PrintDebugString("noecho() == ERR !\n");
	return FAILURE;
    }
    curs_set(0);
    
    HasColors = has_colors();
    if (HasColors != FALSE)
    {
	start_color();
	init_pair(1, COLOR_WHITE, COLOR_BLUE);		// Main
	init_pair(2, COLOR_BLACK, COLOR_CYAN);		// Menu
	init_pair(3, COLOR_BLACK, COLOR_WHITE);		// Status
	init_pair(4, COLOR_BLACK, COLOR_CYAN);		// Sub menu normal
	init_pair(5, COLOR_WHITE, COLOR_BLACK);		// Sub menu selected
	init_pair(6, COLOR_RED, COLOR_WHITE);		// MsgBox
	init_pair(7, COLOR_WHITE, COLOR_BLUE); 		// Bar win
	init_pair(8, COLOR_YELLOW, COLOR_BLUE);		// Data text
	
	// special pairs for SetWindowText
	init_pair(32, COLOR_RED, COLOR_WHITE);
	init_pair(33, COLOR_BLACK, COLOR_WHITE);
	
	// special pairs for ProgressBar
	init_pair(34, COLOR_BLUE, COLOR_BLUE); 		// Bar active
	init_pair(35, COLOR_WHITE, COLOR_BLUE); 	// Bar inactive
    }

    // Initializing all windows and panels
    MainWin = newwin(MAINWIN_ROWS, MAINWIN_COLS, MAINWIN_Y, MAINWIN_X);
    if (MainWin == NULL)
    {
	PrintDebugString("newwin(MainWin) failed!\n");
	return FAILURE;
    }
    MainPanel = new_panel(MainWin);
    show_panel(MainPanel);
    BarWin = newwin(BARWIN_ROWS, BARWIN_COLS, BARWIN_Y, BARWIN_X);
    if (BarWin == NULL)
    {
	PrintDebugString("newwin(BarWin) failed!\n");
	return FAILURE;
    }
    BarPanel = new_panel(BarWin);
    show_panel(BarPanel);

    MenuWin = newwin(MENUWIN_ROWS, MENUWIN_COLS, MENUWIN_Y, MENUWIN_X);
    if (MenuWin == NULL)
    {
	PrintDebugString("newwin(MenuWin) failed!\n");
	return FAILURE;
    }
    MenuPanel = new_panel(MenuWin);
    show_panel(MenuPanel);
    StatusWin = newwin(STATUSWIN_ROWS, STATUSWIN_COLS, 
	STATUSWIN_Y, STATUSWIN_X);
    if (StatusWin == NULL)
    {
	PrintDebugString("newwin(StatusWin) failed!\n");
	return FAILURE;
    }
    StatusPanel = new_panel(StatusWin);
    show_panel(StatusPanel);
    
    keypad(MenuWin, TRUE);
    werase(MainWin);
    werase(BarWin);
    werase(MenuWin);
    werase(StatusWin);

    wattrset(MainWin, COLOR_PAIR(1));
    wattrset(BarWin, COLOR_PAIR(7));
    wattrset(MenuWin, COLOR_PAIR(2));
    wattrset(StatusWin, COLOR_PAIR(3));

    ClearWindow(MainWin, 0, 0, MAINWIN_COLS, MAINWIN_ROWS);
    ClearWindow(BarWin, 0, 0, BARWIN_COLS, BARWIN_ROWS);
    ClearWindow(MenuWin, 0, 0, MENUWIN_COLS, MENUWIN_ROWS);
    ClearWindow(StatusWin, 0, 0, MENUWIN_COLS, MENUWIN_ROWS);

    wborder(MainWin, 0, 0, 0, 0, 0, 0, 0, 0);
    wborder(BarWin, 0, 0, 0, 0, 0, 0, 0, 0);
    
    mvwaddch(StatusWin, 0, 60, ACS_VLINE);
    mvwaddch(StatusWin, 0, 79, ACS_VLINE);
    wattrset(MainWin, COLOR_PAIR(1) | A_BOLD);
    wattrset(BarWin, COLOR_PAIR(1) | A_BOLD);

    mvwprintw(MainWin, 0, 4, MAINWIN_TITLE);
    mvwprintw(BarWin, 0, 4, BARWIN_TITLE);

    wattrset(MainWin, COLOR_PAIR(1));
    wattrset(BarWin, COLOR_PAIR(7));
    wattrset(MenuWin, COLOR_PAIR(2));
    wattrset(StatusWin, COLOR_PAIR(3));

    wrefresh(MainWin);
    wrefresh(BarWin);
    wrefresh(MenuWin);
    wrefresh(StatusWin);

    UpdateAll();
    
    // If Everything is OK
    return SUCCESS;
}

//
// Loading to arrays menu items corresponding to actual horizontal menu pos.
//
ITEM **LoadMenuItems(ITEM **itBuf, int iActualPos)
{
    if (iActualPos == 1)
    {
        *itBuf++ = new_item("Polacz sie z UPS'em\0", "");
        *itBuf++ = new_item("Rozlacz sie z UPS'em\0", "");
        *itBuf++ = new_item("Autotest UPS'a\0", "");
        *itBuf++ = new_item("Wyjscie\0", "");
	*itBuf = NULL;
	return itBuf;
    }
    if (iActualPos == 2)
    {
        *itBuf++ = new_item("O programie\0", "");
	*itBuf = NULL;
	return itBuf;
    }
    return (NULL);
}

static int SubMenuDriver(int cch)
{
    if (cch == '\n' || 
	cch == KEY_EXIT ||
	cch == KEY_LEFT ||
	cch == KEY_RIGHT)
	return (MAX_COMMAND + 1);
    else 
	if (cch == KEY_DOWN)
	    return (REQ_NEXT_ITEM);
	else 
	    if (cch == KEY_UP)
		return (REQ_PREV_ITEM);
	    else 
		if (cch == ' ')
		    return (REQ_TOGGLE_ITEM);
		else
		    return (cch);
}


//
// Display and handle Sub menus (Vertical)
//
int SubMenus(int *iActualPos, int ixCoord)
{
    ITEM	*__item[64];
    ITEM	**item = __item;
    MENU	*Menu;
    WINDOW	*wMenu;
    WINDOW	*dwMenu;
    int		imrows, imcols, icRet;

    /*
     * Allocating and creating
     */    
    item = LoadMenuItems(__item, *iActualPos);
    if (item == NULL)
	return SM_QUITMENU;
    Menu = new_menu(__item);

    scale_menu(Menu, &imrows, &imcols);
    wMenu = newwin(imrows+2, imcols+2, 1, ixCoord);
    wattrset(wMenu, COLOR_PAIR(4));
    ClearWindow(wMenu, 0, 0, imcols+2, imrows+2);
    set_menu_back(Menu, COLOR_PAIR(4));
    set_menu_fore(Menu, COLOR_PAIR(5));

    set_menu_win(Menu, wMenu);
    keypad(wMenu, TRUE);
    box(wMenu, 0, 0);
    dwMenu = derwin(wMenu, imrows, imcols, 1, 1);
    set_menu_sub(Menu, dwMenu);
    post_menu(Menu);

    icRet = -255;
    do
    {
        icRet = wgetch(wMenu);

	if (icRet == KEY_LEFT)
	{
	    *iActualPos = (*iActualPos - 1) % (HMENU_ENTRIES + 1);
	    if (*iActualPos == 0)
		*iActualPos = HMENU_ENTRIES;
	    icRet = SM_CONTINUE;
	    break;
	}
	if (icRet == KEY_RIGHT)
	{
	    *iActualPos = (*iActualPos + 1) % (HMENU_ENTRIES + 1);
	    if (*iActualPos == 0)
	        *iActualPos = 1;
	    icRet = SM_CONTINUE;
	    break;
	}
	if (icRet == '\n')
	{
	    icRet = item_index(current_item(Menu));
	    icRet += ((*iActualPos - 1) * MENU_DIV);
	    break;
	}
    }
    while(menu_driver(Menu, SubMenuDriver(icRet)) != E_UNKNOWN_COMMAND);
    
    if (icRet == -255)
	icRet = SM_QUITMENU;

    /*
     * Cleanup
     */
    unpost_menu(Menu);
    delwin(dwMenu);
    delwin(wMenu);
    free_menu(Menu);
    for(item = __item; *item; item++)
	free_item(*item);

    return (icRet);
}

//
// Display Horizontal menu
//
int DisplayMainMenu(int iActualPos)
{
    int 	i, ilmc, ix;
    const int	iSpaces = 2;
    char	szSpace[10];
    char	szMenuEntries[HMENU_ENTRIES][10] = 
		{ " UPS \0", " Pomoc \0" };

    i = iActualPos;
    ilmc = 1;
    redrawwin(MainWin);
    redrawwin(MenuWin);
    redrawwin(StatusWin);
    // Sets cursor pos

    wattrset(MenuWin, COLOR_PAIR(2));
    mvwprintw(MenuWin, 0, 1, "");
    ix = 1;
    for (i = 1; i <= HMENU_ENTRIES; i++)
    {
	if (iActualPos == i)
	{
	    wattron(MenuWin, A_REVERSE);
	    ilmc = ix;
	}
	wprintw(MenuWin, szMenuEntries[i-1]);
	ix+=strlen(szMenuEntries[i-1]);
	if (iActualPos == i)
	{
	    wattroff(MenuWin, A_REVERSE);
	}
	wprintw(MenuWin, Spaces(szSpace, iSpaces));
	ix+=iSpaces;
    }
    UpdateAll();
    return (ilmc);
}

//
// Clearing content of window using current assigned colors
//
void ClearWindow(WINDOW *win, int lx, int ly, int w, int h)
{
    char	*szSpc;
    int		ih;
    
    szSpc = (char *)calloc(w + 1, sizeof(char));
    for (ih = 0; ih < h; ih++)
	mvwprintw(win, ly + ih, lx, Spaces(szSpc, w));
    free(szSpc);
}

//
// Updating all windows and panels
//
void UpdateAll()
{
    update_panels();
    refresh();
    doupdate();
}

//
// Closing curses before exit to system
//
int CloseCurses()
{
    del_panel(MenuPanel);
    del_panel(MainPanel);
    del_panel(BarPanel);
    del_panel(StatusPanel);
    delwin(MenuWin);
    delwin(MainWin);
    delwin(BarWin);
    delwin(StatusWin);
    werase(stdscr);
    UpdateAll();
    resetty();
    endwin();
    return SUCCESS;
}

char *Spaces(char *szBuf, int in)
{
    int icnt;
    for (icnt = 0; icnt < in; icnt++)
	szBuf[icnt] = ' ';	// 32(Dec)=space
    szBuf[icnt] = '\0';
    return szBuf;
}

//
// initialization of all variables
//
void InitAllVariables()
{
    memset(&UpsInfo, 0, sizeof(UpsInfo));
}

//
// Text displayed when program starts
//
void WelcomeDisplay()
{
    printf(
	"PowerSoft - Panel pomiarowy v%s\n\t%s\n"
	"Kompilacja %s (%s)\n", 
	PRIMARY_VER, PRIMARY_COPYRIGHT, __DATE__, __TIME__);
};

//
// Retrieve Name of UPS
//
char *GetTypeUpsName(UPS_INFO_FIFO *Ups)
{
TRACE;
    switch(Ups->uiUpsType)
    {
    case 67: return "NET 500-DPC";
    case 68: return "NET 700-DPC";
    case 69: return "NET 1000-DPC";
    case 70: return "NET 1400-DPC";
    case 71: return "NET 2200-DPC";
    // Zmiana typow w zwiazku ze zmiana napiec
    case 73: return "NET 700-DPC";
    case 74: return "NET 1000-DPC";
    case 75: return "NET 1400-DPC";
    case 76: return "NET 500-DPC";
    
    case 81: return "AP 450-PRO";
    case 82: return "AP 650-PRO";
    default: return "Brak danych";
    }
}

//
// returns string about current state of UPS
//
char *GetUpsState(UPS_INFO_FIFO *Ups)
{
TRACE;
    if (IsStatusSet(Ups->ulUpsState, BATTERY_WORK))
	return 		"PRACA AWARYJNA! (Brak zasilania)";
    else
	if (IsStatusSet(Ups->ulUpsState, STANDBY_WORK))
	    return	"TRYB OCZEKIWANIA                ";
	else
	    return	"PRACA SIECIOWA (Zasilanie OK)   ";
}

//
// Returns accu. voltage string (float type)
//
char *GetAccuString(char *szBuff, unsigned long ulAccuVal)
{
    float fVal;
    fVal = ulAccuVal / 10.0;
    sprintf(
	szBuff,
	"%.1f",
	fVal);
    return szBuff;
}

//
// Returns current accu. capacity
//
int GetAccuCapacity(UPS_INFO_FIFO Ups)
{
    float fVal;

    if (IsStatusSet(Ups.ulUpsState, BATTERY_WORK))
	fVal = ((((float)Ups.ulUpsAccuValue[0]-95.0)*100.0)/(124.5-95.0));
    else
	if (!IsStatusSet(Ups.ulUpsState, STANDBY_WORK))
	    fVal = ((((float)Ups.ulUpsAccuValue[0]-95.0)*100.0)/(135.5-95.0));
    if (fVal > 100.0)
	fVal = 100.0;
    if (fVal < 0.0)
	fVal = 0.0;
    fVal *= 10.0;
    return ((int)fVal);
}

//
// Returns Accu. capacity string
//
char *GetAccuCapacityString(char *szBuff, UPS_INFO_FIFO Ups)
{
    float fVal, fMul;

    fMul = 1.0;
    
    if (IsStatusSet(Ups.ulUpsState, BATTERY_WORK))
	fVal = ((((float)Ups.ulUpsAccuValue[0]-95.0)*100.0)/(124.5-95.0));
    else
	if (!IsStatusSet(Ups.ulUpsState, STANDBY_WORK))
	    fVal = ((((float)Ups.ulUpsAccuValue[0]-95.0)*100.0)/(135.5-95.0));
    if (fVal > 100.0)
	fVal = 100.0;
    if (fVal < 0.0)
	fVal = 0.0;
    
    sprintf(szBuff, "%.1f", fVal);
    return szBuff;
}

//
// Returns warning string
//
char *Warning(char *szBuff, UPS_INFO_FIFO Ups)
{
    if (IsStatusSet(Ups.ulUpsState, BATTERY_WORK))
	return ("Zachowaj swoja prace! Wyloguj sie. ");
    if ((Ups.ulUpsAccuValue[0] < 105) && 
	IsStatusSet(Ups.ulUpsState, BATTERY_WORK))
	return ("Niski stan naladowania akumulatora!");
    strcpy(
	szBuff, "Nie mam uwag.                      ");
    return     ("Nie mam uwag.                      ");
}

//
// Returns Time (H:M:S) format string
//
char *GetTimeString(char *szBuff, unsigned long ulSec)
{
    int	ih, im, is;
    
    ih = (ulSec / 3600);
    im = ((ulSec - (ih * 3600)) / 60);
    is = ((ulSec - (ih * 3600) - (im * 60)));
    sprintf(szBuff, "%02d:%02d:%02d", ih, im, is);
    return (szBuff);
}

//
// Printing information to Window
//
void PrintWinInfo(WINDOW *wPrint, int ix, int iy, UPS_INFO_FIFO *Ups)
{
    mvwprintw(wPrint, iy+0, ix, 
	"Typ zasilacza");
    mvwprintw(wPrint, iy+1, ix,	
	"Tryb pracy zasilacza");
    mvwprintw(wPrint, iy+2, ix, 
	"Napiecie wejsciowe                [V]");
    mvwprintw(wPrint, iy+3, ix,	
	"Roznica wart. chwilowej nap. wej. [V]");
    mvwprintw(wPrint, iy+4, ix,	
	"Napiecie akumulatora              [V]");
    mvwprintw(wPrint, iy+5, ix,	
	"Pojemnosc akumulatora            [%%]");
    mvwprintw(wPrint, iy+6, ix,	
	"Moc podlaczonego obciazenia      [VA]");
    mvwprintw(wPrint, iy+7, ix,	
	"Czas do zamkniecia systemu    [h:m:s]");
    mvwprintw(wPrint, iy+8, ix,	
	"Czas do wykonania autotestu   [h:m:s]");
    mvwprintw(wPrint, iy+9, ix,	
	"Uwagi!");

    wattrset(wPrint, COLOR_PAIR(8) | A_BOLD);
    mvwprintw(wPrint, iy+0, ix + 41, GetTypeUpsName(&UpsInfoFifo));
    wprintw(wPrint, Spaces(szBuffer, 5));
    mvwprintw(wPrint, iy+1, ix + 41, GetUpsState(&UpsInfoFifo));
    mvwprintw(wPrint, iy+2, ix + 41, "%d  ", (UpsInfoFifo.ulUpsACValue[0] + 
	UpsInfoFifo.ulUpsACValue[1] + UpsInfoFifo.ulUpsACValue[2]) / 3);
    mvwprintw(wPrint, iy+3, ix + 41, "%d  ", UpsInfoFifo.ulUpsACDifference);
    mvwprintw(wPrint, iy+4, ix + 41, "%s  ",
	GetAccuString(szBuffer, UpsInfoFifo.ulUpsAccuValue[0]));
    mvwprintw(wPrint, iy+5, ix + 41, "%s  ", 
	GetAccuCapacityString(szBuffer, UpsInfoFifo));
    mvwprintw(wPrint, iy+6, ix + 41, "%d  ", 
	(int)((float)UpsInfoFifo.ulUpsPowerValue[0] * 1.666));
    mvwprintw(wPrint, iy+7, ix + 41, "%s  ", 
	GetTimeString(szBuffer, UpsInfoFifo.ulTimeToShutdown1));
    mvwprintw(wPrint, iy+8, ix + 41, "%s  ", 
	GetTimeString(szBuffer, UpsInfoFifo.ulTimeToAutotest1));
    mvwprintw(wPrint, iy+9, ix + 41, "%s  ", Warning(szBuffer, UpsInfoFifo));

    wattrset(wPrint, COLOR_PAIR(1));

    mvwaddch(MainWin, 0, 40, ACS_TTEE);
    mvwvline(MainWin, 1, 40, ACS_VLINE, 11);
    mvwaddch(MainWin, 12, 40, ACS_BTEE);
    redrawwin(MainWin);
}

//
// Terminate program from signal
//
void Terminate(int iSig)
{
    CloseCurses();
    printf("Termination on signal(%d)!\n", iSig);
    sleep(1);
    _exit(0);
}

//
// Terminate threads
//
void TerminateThread()
{
    CloseCurses();
    RestoreSystemSignals();
    _exit(0);
}

//
// Display About box
//
void About()
{
    WINDOW	*CWin;
    PANEL	*CPanel;

    CWin = newwin(14, 50, 5, 15);
    if (CWin == NULL)
	return;
    CPanel = new_panel(CWin);
    show_panel(CPanel);
    werase(CWin);
    wattrset(CWin, COLOR_PAIR(6));
    ClearWindow(CWin, 0, 0, 50, 14);
    
    wborder(CWin, 0, 0, 0, 0, 0, 0, 0, 0);
    wrefresh(CWin);
    
    mvwprintw(CWin, 1, 2, "PowerSoft - Panel kontrolny (Beta)           ");
    mvwaddch(CWin, 2, 0, ACS_LTEE);
    mvwhline(CWin, 2, 1, ACS_HLINE, 48);
    mvwaddch(CWin, 2, 49, ACS_RTEE);
    mvwprintw(CWin, 3, 3, "Autor: Tomasz Fiszer <tfiszer@ever.com.pl>   ");
    mvwprintw(CWin, 4, 3, "Strona WWW: www.ever.com.pl                  ");
    mvwprintw(CWin, 5, 3, "Prawa autorskie (C) 2001-2002 EVER Sp. z o.o.");
    mvwaddch(CWin, 6, 0, ACS_LTEE);
    mvwhline(CWin, 6, 1, ACS_HLINE, 48);
    mvwaddch(CWin, 6, 49, ACS_RTEE);
    mvwprintw(CWin, 7, 3, "Program kontrolny do podgladu parametrow     ");
    mvwprintw(CWin, 8, 3, "pracy zasilaczy awaryjnych firmy EVER.       ");
    mvwaddch(CWin, 11, 0, ACS_LTEE);
    mvwhline(CWin, 11, 1, ACS_HLINE, 48);
    mvwaddch(CWin, 11, 49, ACS_RTEE);
    mvwprintw(CWin, 12, 2, "EVER Sp. z o.o. - Swarzedz, Polska          ");

    UpdateAll();
    wgetch(CWin);
    del_panel(CPanel);
    delwin(CWin);
}

//
// Displaying message box
//
void MsgBox(char *szBuf)
{
    WINDOW	*CWin;
    PANEL	*CPanel;
    char	szwStr[256];
    int		ilenght;

    strcpy(szwStr, szBuf);
    if (strlen(szwStr) >= 30)
    {
	ilenght = strlen(szwStr) + 4;
	CWin = newwin(5, ilenght, 10, (COLS / 2) - (ilenght / 2));
    }
    else
    {
	CWin = newwin(5, 30, 10, 24);
	ilenght = 30;
    }

    if (CWin == NULL)
	return;
    CPanel = new_panel(CWin);
    show_panel(CPanel);
    werase(CWin);
    wattrset(CWin, COLOR_PAIR(6));
    ClearWindow(CWin, 0, 0, ilenght, 5);
    
    wborder(CWin, 0, 0, 0, 0, 0, 0, 0, 0);
    wrefresh(CWin);

    mvwprintw(CWin, 1, 2, szwStr);
    mvwprintw(CWin, 3, 2, "Nacisnij dowolny klawisz...");

    UpdateAll();
    wgetch(CWin);
    del_panel(CPanel);
    delwin(CWin);
}

//
// Printing text to status box
//
void SetStatusText(char *szText)
{
    char	szBuff[20], 
		szSpc[20],
		szBuffOut[22];
    
    if (strlen(szText) <= 18)
	strcpy(szBuff, szText);
    else
	strncpy(szBuff, szText, 18);
    strcpy(szBuffOut, "^");
    strcat(szBuffOut, szBuff);
    strcat(szBuffOut, "^");
    strcat(szBuffOut, Spaces(szSpc, 18 - strlen(szText)));

    SetWindowText(StatusWin, 61, 0, szBuffOut);
}

//
// Connection to UPS via FIFO
//
void ConnectToUPS()
{
    WINDOW	*CWin;
    PANEL	*CPanel;
    char	szwStr[256];
    pid_t	pPid;
    FILE	*fPid;
    int		iError = 0;
    
    // Checking for PowerSoft existence
    fPid = fopen(POWERSOFTPID, "r");
    if (fPid != NULL)
    {
	fread(szwStr, sizeof(char), 16, fPid);
	pPid = atol(szwStr);
        if ((kill(pPid, 0) == -1) && (errno == ESRCH))
	    iError = 1;
	fclose(fPid);
    }
    else
	iError = 1;
    if (iError == 1)
    {
	SetStatusText("Nie polaczony");
	MsgBox("PowerSoft nie jest uruchomiony!");
	iConnected = FALSE;
	return;
    }

    iConnected = TRUE;
    CWin = newwin(3, 30, 10, 25);
    if (CWin == NULL)
	return;
    CPanel = new_panel(CWin);
    show_panel(CPanel);
    werase(CWin);
    wattrset(CWin, COLOR_PAIR(3));
    ClearWindow(CWin, 0, 0, 30, 3);
    wborder(CWin, 0, 0, 0, 0, 0, 0, 0, 0);
    wrefresh(CWin);
    strcpy(szwStr, "Laczenie...");
    mvwprintw(CWin, 1, 15 - (strlen(szwStr) / 2), szwStr);
    UpdateAll();
    MainThread(0);
    del_panel(CPanel);
    delwin(CWin);
    SetStatusText("Polaczony");
    PrintWinInfo(MainWin, 1, 1, &UpsInfoFifo);
    
    signal(SIGALRM, MainThread);
    alarm(4);
}

//
// Thread
//
void MainThread(int iSig)
{
    bLoopBusy = TRUE;
    if (ReadFifo(&UpsInfoFifo, 0) == SUCCESS_TWOSIDE)
    {
	// This Must be do
	UpsInfoFifo.ulOperation = 0;
	FifoExchange(&UpsInfoFifo, 0);
    }
    PrintWinInfo(MainWin, 1, 1, &UpsInfoFifo);
    
    wattrset(BarWin, COLOR_PAIR(7));
    mvwprintw(BarWin, 2, 2, "Napiecie sieciowe");
    mvwprintw(BarWin, 3, 73, "[V]");
    mvwprintw(BarWin, 4, 2, 
	"0               67               "
	"135              202              270");
    mvwprintw(BarWin, 5, 2, "Pojemnosc akumulatora");
    mvwprintw(BarWin, 6, 73, "[%%]");
    mvwprintw(BarWin, 7, 2, 
	"0               25               "
	"50               75               100");
    ProgressBar(BarWin, 2, 3, 70, 1, BAR_HTYPE,
	270, (int)(UpsInfoFifo.ulUpsACValue[0] + UpsInfoFifo.ulUpsACValue[1] +
	UpsInfoFifo.ulUpsACValue[2]) / 3);
    ProgressBar(BarWin, 2, 6, 70, 1, BAR_HTYPE, 
	100, (GetAccuCapacity(UpsInfoFifo) / 10));

    RefreshWindows();
    redrawwin(MainWin);
    redrawwin(BarWin);
    redrawwin(StatusWin);
    signal(SIGALRM, MainThread);
    alarm(4);
    bLoopBusy = FALSE;
    return;
}

//
// Progress Bar
//
void ProgressBar(
	WINDOW *Win, 				// Window identifier
	int ix, int iy, int il, int iw, 	// Bar coordinates
	int iType, 				// Bar Type ( Horiz, Vert )
	int iMax, int iVal)			// Bar values to display
{
    // Local variables
    int		iCharBar = ACS_BLOCK;
    int		icnt, icntw;
    int 	iValue;
    short	swattr[3];

    icnt = icntw = iValue = 0;
    wattr_get(Win, &swattr[0], &swattr[1], &swattr[2]);
    iValue = (int)(float)(((((float) iVal * 100.0) / 
		(float) iMax) / 100.0) * (float) il);
    
    if (iType == BAR_HTYPE)
    {
        for (icnt = 1; icnt < il; icnt++)
	{
	    if (icnt <= iValue)
		wattrset(Win, COLOR_PAIR(34) | A_BOLD);
	    else
		wattrset(Win, COLOR_PAIR(35) | A_BOLD);
	    for (icntw = 0; icntw < iw; icntw++)
		mvwaddch(Win, iy + icntw, ix + icnt - 1, iCharBar);
	}
    }
    if (iType == BAR_VTYPE)
    {
        for (icnt = 1; icnt < il; icnt++)
	{
	    if (icnt <= iValue)
		wattrset(Win, COLOR_PAIR(34));
	    else
		wattrset(Win, COLOR_PAIR(35) | A_BOLD);
		
	    for (icntw = 0; icntw < iw; icntw++)
		mvwaddch(Win, iy + icnt - 1, ix + icntw, iCharBar);
	}
    }
    
    wattrset(Win, swattr[2]);
}

//
// Printing text to window
//
void SetWindowText(WINDOW *Win, int x, int y, char *szString)
{
    char	*szBuff;
    int		icnt, ilen, ihl, idiff;
    
    ihl = FALSE;
    idiff = 0;
    ilen = strlen(szString);
    szBuff = (char *)calloc(ilen + 1, sizeof(char));
    if (szBuff != NULL)
    {
	strcpy(szBuff, szString);
	wattrset(Win, COLOR_PAIR(33));
	
	for (icnt = 0; icnt < ilen; icnt++)
	{
	    if (szBuff[icnt] == '^' && ihl == FALSE)
	    {
		ihl = TRUE;
		wattrset(Win, COLOR_PAIR(32));
		idiff++;
		continue;
	    }
	    else
		if (szBuff[icnt] == '^' && ihl == TRUE)
		{
		    ihl = FALSE;
		    wattrset(Win, COLOR_PAIR(33));
		    idiff++;
		    continue;
		}
		else
		{
		    mvwaddch(Win, y, x + icnt - idiff, szBuff[icnt]);
		}
	}
	
	free(szBuff);
    }
    else
    {
	MsgBox("SetWindowText: Blad podczas alokacji pamieci!");
	Terminate(0);
    }
}

//
// Autotesting UPS parameters
//
void Autotest()
{
    WINDOW	*CWin;
    PANEL	*CPanel;
    char	szwStr[256];
    pid_t	pPid;
    FILE	*fPid;
    int		iError = 0;
    
    // Checking if PowerSoft exists
    fPid = fopen(POWERSOFTPID, "r");
    if (fPid != NULL)
    {
	fread(szwStr, sizeof(char), 16, fPid);
	pPid = atol(szwStr);
        if ((kill(pPid, 0) == -1) && (errno == ESRCH))
	    iError = 1;
	fclose(fPid);
    }
    else
	iError = 1;
    if (iError == 1)
    {
	MsgBox("PowerSoft nie jest uruchomiony!");
	return;
    }

    //echo();
    CWin = newwin(3, 30, 10, 25);
    if (CWin == NULL)
	return;
    CPanel = new_panel(CWin);
    show_panel(CPanel);
    werase(CWin);
    wattrset(CWin, COLOR_PAIR(3));
    ClearWindow(CWin, 0, 0, 30, 3);
    wborder(CWin, 0, 0, 0, 0, 0, 0, 0, 0);
    wrefresh(CWin);
    strcpy(szwStr, "Laczenie...");
    mvwprintw(CWin, 1, 15 - (strlen(szwStr) / 2), szwStr);
    UpdateAll();

    if (ReadFifo(&UpsInfoFifo, 0) == SUCCESS_TWOSIDE)
    {
	memset(&UpsInfoFifo, 0, sizeof(UPS_INFO_FIFO));
	UpsInfoFifo.ulOperation = FIFO_AUTOTEST;
	FifoExchange(&UpsInfoFifo, 0);
    }

    del_panel(CPanel);
    delwin(CWin);
}

//
// Executing menu command
//
int ExecuteCommand(int iIndex)
{
    //
    // If 1 or more is returned then program ends
    //
    switch(iIndex)
    {
	case 0:
	    if (iConnected != TRUE)
		ConnectToUPS();
	    else
		MsgBox("Polaczenie jest juz aktywne!");
	    break;
	case 1:
	    if (iConnected == TRUE)
		alarm(0);
	    else
		MsgBox("Polaczenie jest nieaktywne!");
	    SetStatusText("Nie polaczony");
	    iConnected = FALSE;
	    break;
	case 2:
	    if (iConnected == TRUE)
	    {
		alarm(0);
		Autotest();
		alarm(4);
	    }
	    else
		Autotest();
	    break;
	// Only here we can return (>= 1)
	// Used to execute 'Exit' command from sub menu
	// Returned value is passed to (iExit) variable
	case 3:
	    return 1;
	
	// Menu number 2;
	case (0 + MENU_DIV):
	    About();
	    break;
    }
    return 0;
}

//
// Main function
//
int main(int argc, char** argv)
{
    int	iError, i, iExit, iMenuPos, iMenuEntered, iLastMenuCoord, iSubRet;
    i = iError = 0;

    memset(&UpsInfo, 0, sizeof(UPS_INFO));
    memset(&UpsInfoFifo, 0, sizeof(UPS_INFO_FIFO));
    WelcomeDisplay();
    // To see WelcomeMessage
    if (!CheckSuperUser()) 
	_exit(0); // Exit here because we need "Root" access rights

    InitAllVariables();
    if (InitGraphics() == FAILURE)
    {
	CloseCurses();
	PrintError("Cannot initialize ncurses screen!\n", 0, 0);
	return EXIT_FAILURE;
    }
    InitSystemSignals();
    iExit = 0;
    iMenuPos = 0;
    iMenuEntered = 0;
    iLastMenuCoord = 0;
    SetWindowText(StatusWin, 2, 0, 
	"^F1^-Pomoc ^F2^-Polacz/Rozlacz ^F9^-Menu ^F10^-Wyjscie");
    SetStatusText("Nie polaczony");
    ExecuteCommand(0);
    
    while(!iExit)
    {
	iLastMenuCoord = DisplayMainMenu(iMenuPos);
	switch (wgetch(MenuWin))
	{
	    case KEY_F(1):
		ExecuteCommand(0 + MENU_DIV);
		break;

	    case KEY_F(2):
		if (iConnected == FALSE)
		    ExecuteCommand(0);
		else
		    ExecuteCommand(1);
		break;
	
	    case KEY_F(5):
		//while(bLoopBusy)
		//    sleep(2);
		RefreshWindows();
		break;

	    case KEY_F(9):
		if (iMenuEntered == 0)
		{
		    iMenuPos = iMenuEntered = 1;
		    if (iConnected == TRUE)
			alarm(0);
		}
		break;

	    /* ENTER OR DOWN ARROW */
	    case KEY_DOWN:
	    case 10:
		if (iMenuEntered == 1)
		{
		SubMenuLoop:
		    iSubRet = SubMenus(&iMenuPos, iLastMenuCoord);
		    switch(iSubRet)
		    {
			case SM_CONTINUE:
			    iLastMenuCoord = DisplayMainMenu(iMenuPos);
			    redrawwin(MainWin);
			    UpdateAll();
			    goto SubMenuLoop;
			default: 
			    iExit = ExecuteCommand(iSubRet);
			case SM_QUITMENU:
			    iMenuEntered = 0;
			    iMenuPos = 0;
			    break;
		    }
		}
		break;
		
	    case KEY_LEFT:
		if (iMenuEntered == 0)
		    iMenuPos = 0;
		else
		{
		    iMenuPos = (iMenuPos - 1) % (HMENU_ENTRIES + 1);
		    if (iMenuPos == 0)
			iMenuPos = HMENU_ENTRIES;
		}
		break;
		
	    case KEY_RIGHT:
		if (iMenuEntered == 0)
		    iMenuPos = 0;
		else
		{
		    iMenuPos = (iMenuPos + 1) % (HMENU_ENTRIES + 1);
		    if (iMenuPos == 0)
			iMenuPos = 1;
		}
		break;
		
	    /* ESCAPE */
	    case 27:
		if (iMenuEntered == 1)
		{
		    iMenuEntered = iMenuPos = 0;
		    if (iConnected == TRUE)
			alarm(4);
		}
		break;
		
	    case KEY_F(10):
		if (iMenuEntered == 1)
		{
		    iMenuEntered = iMenuPos = 0;
		    if (iConnected == TRUE)
			alarm(4);
		}
		else
		    iExit = 1;
		break;
	}
    }
    CloseCurses();
    return EXIT_SUCCESS;
}
